import { controller } from "@/components/treeview/signals";
import { globalLookup } from "./global";
import { WaveContainerView } from "./wave-container-view";
import { isScope, isVariable } from "./utils";

export const recoverConfig = {
    /**
     * @type {Map<string, IRenderOption>} 其中的 string 是形如 `cpu.alu.add1` 这样的字符串
     */
    waves: new Map(),
};

async function findRecoverFile(recoverJsonPath) {
    try {
        const response = await fetch(recoverJsonPath);
        if (!response.ok) {
            return undefined;
        }
        const recoverJson = await response.json();
        return recoverJson;
    } catch (error) {
        return undefined;
    }
}

/**
 * 
 * @param {string} inputFile 读取的 vcd 文件的路径
 */
export async function recoverFromInputFile(inputFile) {
    const recoverJsonPath = inputFile + '.pkl';
    const recoverJson = await findRecoverFile(recoverJsonPath);
    if (recoverJson) {
        const waves = recoverJson.waves;
        if (waves instanceof Array && waves.length > 0) {
            for (const wave of waves) {
                const name = wave.name;
                const option = wave.option;
                if (name === undefined || option === undefined || recoverConfig.waves.has(name)) {
                    continue;
                }
                recoverConfig.waves.set(name, option);
            }
        }
    }
}

class PrefixNode {
    constructor() {
        this.children = new Map();
        this.payload = undefined;
    }
}



/**
 * @description 恢复现场的函数，主要恢复两个变量 currentWiresRenderView 和 currentSignalRenderOptions
 * @param {TopModWireItem[]} topModules
 */
export function recoverSession(topModules) {
    const waves = recoverConfig.waves;
    if (waves.size === 0) {
        return;
    }

    // 利用前缀树 记忆化搜索
    for (const [name, option] of waves.entries()) {
        const deps = name.split('.');
        let layer = 0;
        let nodes = topModules;
        let result = undefined;
        let lastNode = undefined;

        while (layer < deps.length) {
            // 从当前 nodes 中匹配 className
            const targetName = deps[layer];
            let targetNode = nodes.filter(n => n.name == targetName)[0];
            if (targetNode === undefined) {
                break;
            }

            if (isScope(targetNode)) {
                nodes = targetNode.body;
            } else if (isVariable(targetNode)) {
                result = targetNode;
                if (result.parent === undefined) {
                    result.parent = lastNode;
                }
            } else {
                break;
            }
            lastNode = targetNode;
            layer ++;
        }

        if (result !== undefined) {
            // 找到了
            
            WaveContainerView.add(result);
            globalLookup.currentSignalRenderOptions.set(result.link, option);
            controller.lastSignal = result;
        }
    }

    globalLookup.render();
}